################################################################################
###RNAseq data
################################################################################


###Set your work directory
# setwd("E:/OneDrive - Wageningen University & Research/Shared_data_Mark/Projects/Qi_Zheng/Qi_RNAseq/R")
# setwd("C:/Users/sterk009/OneDrive - Wageningen University & Research/Shared_data_Mark/Projects/Qi_Zheng/Qi_RNAseq/R")
# workwd <- getwd()

###Load pre-made functions
    #uses eQTL pipeline functions https://git.wur.nl/mark_sterken/eQTL_pipeline
    #    transcriptomics functions https://git.wur.nl/mark_sterken/Transcriptomics.workbench
    # git_dir <- "E:/OneDrive - Wageningen University & Research/Shared_data_Mark/Projects_R_Zone/Git"
    # source(paste(git_dir,"/NEMA_functions/Loader.R",sep=""))
    # 
    # NEMA.get.functions("Analysis.R")

    #Load custom functions
    for(i in 1:length(dir("./Function_scripts/"))){
        source(paste("./Function_scripts/", dir("./Function_scripts/")[i], sep = ""))
    }

###Set data locations
resources_dir <- paste(git_dir,"/../R_Resources/",sep="")


################################################################################
###Dependencies
################################################################################

install <- FALSE
#.libPaths(.libPaths("C:/Program Files/R/R-3.3.3/library"))
if(install){
    install.packages("tidyverse")
    install.packages("colorspace")
    install.packages("RColorBrewer")
    install.packages("BiocInstaller",repos="http://bioconductor.org/packages/3.3/bioc")
    #source("http://www.bioconductor.org/biocLite.R") ; biocLite("limma") ; biocLite("statmod")
    install.packages("gridExtra")
    install.packages("VennDiagram")
    install.packages("openxlsx")
    install.packages("rmarkdown")
    install.packages("broom")
}

###load
    library("colorspace")
    library("RColorBrewer")
    library(limma)
    library(gridExtra)
    library("VennDiagram")
    library(openxlsx)
    library("rmarkdown")
    library(tidyverse)
    library(broom)


################################################################################
###Plotting theme, colours
################################################################################


###Set plotting theme
    presentation <- theme(axis.text.x = element_text(size=12, face="bold", color="black"),
                      axis.text.y = element_text(size=12, face="bold", color="black"),
                      axis.title.x = element_text(size=14, face="bold", color="black"),
                      axis.title.y = element_text(size=14, face="bold", color="black"),
                      strip.text.x = element_text(size=14, face="bold", color="black"),
                      strip.text.y = element_text(size=14, face="bold", color="black"),
                      plot.title = element_text(size=24, face="bold"),
                      panel.background = element_rect(fill = "white",color="black"),
                      panel.grid.major = element_line(colour = "grey80"),
                      panel.grid.minor = element_blank(),
                      legend.position = "right")

    blank_theme <- theme(plot.background = element_blank(),
                         panel.grid.major = element_blank(),
                         panel.grid.minor = element_blank(),
                         panel.border = element_blank(),
                         panel.background = element_blank(),
                         axis.title.x = element_blank(),
                         axis.title.y = element_blank(),
                         axis.text.x = element_blank(),
                         axis.text.y = element_blank(),
                         axis.ticks = element_blank())


###Here you can set colours for plotting in theme using ggplot2
    #display.brewer.all()

    myColors <- c(brewer.pal(9,"Set1")[c(2,5)],brewer.pal(12,"Paired")[c(1,2,7,8,8)],"black","grey")
    
    names(myColors) <- c("D383","Rookmaker","D383\n(3 days)","Rookmaker\n(3 days)","D383\n(6 days)","Rookmaker\n(6 days)","TRUE","FALSE","MQ")

    colScale <- scale_colour_manual(name = "Treatment",values = myColors)
    fillScale <- scale_fill_manual(name = "Treatment",values = myColors)
    

###################################################################
###RNAseq analysis
################################################################################
    
    ################################################################################
    ###RNAseq: load data
    ################################################################################
        
        ###normalization / QC
        if(FALSE){
             source("./scripts/normalize_QC.R")
        } else {
            load(file="./obj_gen.id.Rdata")
            load(file="./obj_list.data.tmp.Rdata")
            
            list.data.clean <- mutate(list.data,Nematode=ifelse(Nematode=="MQ","Mock",Nematode))
        }
    
    
    ################################################################################
    ###RNAseq: correlation analysis (just for checking)
    ################################################################################

           # ###Potato        
           #  data.test <- group_by(list.data.clean,gene_id) %>%
           #               dplyr::select(SampleID,gene_id,FPKM_ratio_mean) %>%
           #               spread(key=SampleID,value=FPKM_ratio_mean)
           # 
           #  data.cor <- cor(data.matrix(data.test[,-1]))
           #  colscale <- color.scale(colors.use=brewer.pal(11,"PiYG"),y.val=as.numeric(data.cor))
           # 
           #  ###Save the figure
           #  pdf(file="Figure1_correlation.pdf",width=16,height=8)#,units="in",compression="lzw+p",res=600)
           #      heatmap(data.cor,scale="none",col=colscale,margins=c(10,8))
           #      par(fig=c(0,0.3,0.5,1),new=T)
           #      scale.legend(input=as.numeric(data.cor),start.zero=F,col.scale=colscale,line.col="black")
           #  dev.off()
           # 

    ################################################################################
    ###RNAseq: Principal component analysis (figure 1B)
    ################################################################################
    
        list.data.clean <- group_by(list.data.clean,gene_id,replicate) %>%
                           mutate(FPKM_ratio_mean=log2(FPKM/mean(FPKM)))
            
        ###PCA
        data.test <- group_by(list.data.clean,gene_id) %>%
                     dplyr::mutate(sample=paste(SampleID,dpi,replicate,Nematode,sep=":")) %>%
                     dplyr::select(sample,gene_id,FPKM_ratio_mean) %>%
                     spread(key=sample,value=FPKM_ratio_mean)

        # tmp <- prcomp(t(data.matrix(data.test[,-1])),scale. = TRUE)
        #     summary(tmp)
        # 
        # tool.extrnames <- function(x,y){a <- unlist(strsplit(x,split=":"));return(a[seq(y,length(a),by=length(a)/length(x))])}
        #     
        # plot.data <- as.data.frame(cbind(SampleID=rownames(tmp$x),replicate=tool.extrnames(rownames(tmp$x),3),treatment=tool.extrnames(rownames(tmp$x),4),
        #                                  timepoint=tool.extrnames(rownames(tmp$x),2),tmp$x))

        tmp <- prcomp(data.matrix(data.test[,-1]),scale. = TRUE)
            summary(tmp)

        tool.extrnames <- function(x,y){a <- unlist(strsplit(x,split=":"));return(a[seq(y,length(a),by=length(a)/length(x))])}
            
        plot.data <- as.data.frame(cbind(SampleID=rownames(tmp$rotation),replicate=tool.extrnames(rownames(tmp$rotation),3),treatment=tool.extrnames(rownames(tmp$rotation),4),
                                         timepoint=tool.extrnames(rownames(tmp$rotation),2),tmp$rotation))


        for(i in 5:ncol(plot.data)){
            plot.data[,i] <- as.numeric(as.character(unlist(plot.data[,i])))
        }


        ###check if there are clear/weird splits on pc1-6. only 1 and 2 are sufficient for exp. var.
        
        # ###plot of pca1 and 2
        #     p1 <- ggplot(plot.data, aes(x=PC1,y=PC2,colour=treatment,shape=timepoint,label=replicate)) + 
        #           geom_point() + #annotate("text",x=c(0,0,-0.1,0.15),y=c(-0.15,0.175,0.05,0.1),label=c("M. incognita","mock","late","early")) + 
        #           presentation + xlab("PCO 1 (32.4%)") + ylab("PCO 2 (17.5%)") + ggrepel::geom_text_repel() +
        #           theme(legend.position = "none")
        #         
        # ###plot of pca3 and 4    
        #     p2 <- ggplot(plot.data, aes(x=PC3,y=PC4,colour=treatment,shape=timepoint,label=replicate)) + 
        #           geom_point() + #annotate("text",x=c(0,0,-0.1,0.15),y=c(-0.15,0.175,0.05,0.1),label=c("M. incognita","mock","late","early")) + 
        #           presentation + xlab("PCO 3 (11.4%)") + ylab("PCO 4 (8.9%)") + ggrepel::geom_text_repel() +
        #           theme(legend.position = "none")                 
        # 
        # ###plot of pca5 and 6    
        #     p3 <- ggplot(plot.data, aes(x=PC5,y=PC6,colour=treatment,shape=timepoint,label=replicate)) + 
        #           geom_point() + #annotate("text",x=c(0,0,-0.1,0.15),y=c(-0.15,0.175,0.05,0.1),label=c("M. incognita","mock","late","early")) + 
        #           presentation + xlab("PCO 5 (5.7%)") + ylab("PCO 6 (4.6%)") + ggrepel::geom_text_repel() +
        #           theme(legend.position = "right")      
        #     
        # 
        # pdf(file="Figure_PCA_1to6.pdf",width=10,height=3)#,units="in",compression="lzw+p",res=600)
        #     grid.arrange(p1,p2,p3,ncol=3,widths = c(1,1,1.5))    
        # dev.off()

            
        myColors <- c(brewer.pal(9,"Set1")[c(2,5)],"grey")
        
        names(myColors) <- c("D383","Rookmaker","Mock")
    
        colScalenu <- scale_colour_manual(name = "Treatment",values = myColors)
        fillScalenu <- scale_fill_manual(name = "Treatment",values = myColors)
        
        p1 <- ggplot(plot.data, aes(x=PC1,y=PC2,colour=treatment,shape=timepoint,label=replicate)) + 
              geom_point() + #annotate("text",x=c(0,0,-0.1,0.15),y=c(-0.15,0.175,0.05,0.1),label=c("M. incognita","mock","late","early")) + 
              presentation + xlab("PCO 1 (32.4%)") + ylab("PCO 2 (17.5%)") + colScalenu +
              theme(legend.position = "right")                
        
        pdf(file="Figure1B_PCA12.pdf",width=4.5,height=3)#,units="in",compression="lzw+p",res=600)
            p1
        dev.off()
                
    ################################################################################
    ###RNAseq: Differentially expressed genes (R step only)
    ################################################################################
    
        # data.test <- group_by(list.data.clean,gene_id ) %>%
        #              dplyr::mutate(sample = paste(Nematode,dpi,replicate,sep=":")) %>%
        #              dplyr::select(sample,gene_id,FPKM_log) %>%
        #              spread(key=sample,value=FPKM_log)
        #         
        # samples <- data.frame(cbind(sample=colnames(data.test)[-1])) %>%
        #            tidyr::separate(sample,into=c("Nematode","dpi","replicate"),sep=":") %>%
        #            mutate(dpi=as.numeric(as.character(unlist(dpi))),
        #                   Nematode=factor(Nematode,levels = c("MQ","D383","Rookmaker")),
        #                   replicate=as.factor(replicate)) %>%
        #            mutate(infection=ifelse(Nematode=="MQ",0,1),D383=ifelse(Nematode=="D383",1,0),Rook=ifelse(Nematode=="Rookmaker",1,0),MQ=ifelse(Nematode=="MQ",1,0))
        # 
        # genes <- data.test$gene_id
        #             
        # data.test <- data.matrix(data.test[,-1])
        #     rownames(data.test) <- genes
        # 
        #     
        # ###Tried MQ vs D383/Rook models and the integrated model below. Model below performs well and identifies genes
        # ###differential expressed between one strain and the rest.
        # 
        # test.nu <- samples[,2]== 6 & samples[,1] %in% c("D383","Rookmaker","MQ")   
        # 
        # tmp <- transcriptomics.lm(lm_data = data.test[,test.nu],lm_factors = samples[test.nu,c(5,6)])$model %>%
        #        as_tibble(.name_repair = "universal",rownames="gene_id") %>%
        #        mutate(dpi=unique(samples[test.nu,2]))
        #        
        # output <- tmp
        # 
        # 
        # test.nu <- samples[,2]== 3 & samples[,1] %in% c("D383","Rookmaker","MQ")   
        # 
        # tmp <- transcriptomics.lm(lm_data = data.test[,test.nu],lm_factors = samples[test.nu,c(5,6)])$model %>%
        #        as_tibble(.name_repair = "universal",rownames="gene_id") %>%
        #        mutate(dpi=unique(samples[test.nu,2]))
        # 
        # output <- rbind(output,tmp)
        # 
        # output2 <- rbind(cbind(dpi=output[,6],Comparison="D383",gene_id=output[,1],significance=unlist(output[,2]),effect=unlist(output[,4])),
        #                  cbind(dpi=output[,6],Comparison="Rookmaker",gene_id=output[,1],significance=unlist(output[,3]),effect=unlist(output[,5])))
        # 
        # output <- output2
        # 
        # model.data <- data.frame(output)
        # for(i in c(1,4,5)){model.data[,i] <- as.numeric(as.character(unlist(model.data[,i])))}
        # for(i in c(2,3)){model.data[,i] <- as.character(unlist(model.data[,i]))}                                
        # 
        # model.data <- group_by(model.data,Comparison,dpi) %>%
        #               mutate(fdr=p.adjust(significance,method = "fdr"),Bonf=p.adjust(significance,method = "bonferroni")) %>%
        #               data.frame() %>%
        #               merge(gene.id)
        # 
        # save(model.data,file="obj_model.data.Rdata")
        
        
    ################################################################################
    ###RNAseq: Differentially expressed genes (Supplementary figure 1; supplementary table 3)
    ################################################################################
        
        load(file="./data/obj_model.data.Rdata")
        
        ###We set a threshold at -log10(p) > 3; this results in fdr < 0.05 for each of the comparisons. 
        ###this is preferable as it makes effect sizes similar over all comparisons
        filter(model.data,significance<0.001) %>%
        group_by(Comparison,dpi) %>%
        summarise(thr=max(fdr),n=length(effect))

        pdf(file="Supplementary_figure1-volcano_plot.pdf",width=5,height=4.5)#,units="in",compression="lzw+p",res=600)
            ggplot(model.data,aes(x=effect,y=-log10(significance),colour=significance<0.001)) +
            geom_point(alpha=0.3) + facet_grid(dpi~Comparison) + geom_hline(yintercept = 3,lty=2) +
            presentation + colScale + labs(x=expression(bold(log[2](effect))),y=expression(bold(-log[10](significance)))) +
            theme(legend.position = "none")
        dev.off()
        
        data.table <- filter(model.data,significance<0.001)
        
        ###Check how many genes
        length(unique(data.table$gene_id))
        
        write.xlsx(data.table,file="Supplementary_table3-DEGs.xlsx")
        
            
            
    ################################################################################
    ###RNAseq: Venn diagram (figure 1C; Supplementary table 4)
    ################################################################################
            
        ###Venn diagram of overlaps    
        data.plot <- filter(model.data,significance<0.001) %>%   #abs(effect)>2
                     mutate(plottie=paste(Comparison,"\n(",dpi," days)",sep="")) %>%
                     dplyr::select(gene_id,plottie)
                    
        data.plot <- split(data.plot[,1],data.plot[,2])
        
        venn.RIL <- venn.diagram(data.plot,filename = NULL, fill=myColors[c(3,4,5,6)],sub.fontfamily = "sans", main.fontfamily = "sans",cat.fontfamily="sans",fontfamily="sans")
        
        tiff(file="Figure1C-Venn_DEG.tif",width=4,height=4,units="in",compression="lzw+p",res=600)  
            grid.newpage()
            grid.draw(venn.RIL)
        dev.off()  
        
        
        data.table <- filter(model.data,significance<0.001) %>%   #,abs(effect)>2
                      mutate(plottie=paste(Comparison,dpi,sep="_")) %>%
                      dplyr::select(gene_id,effect,plottie,gene_name:tf_family) %>%
                      spread(key=plottie,value=effect) %>%
                      arrange(is.na(D383_3),is.na(D383_6),is.na(Rookmaker_3),is.na(Rookmaker_6))
                      
        writexl::write_xlsx(data.table,path="Supplementary_table4_Venn_genes.xlsx")
            

    ################################################################################
    ###RNAseq: overview by k-mean clustering (figure 1D; supplementary table 5)
    ################################################################################
        
        # ###add k-means pattern
        # ##heatmap plot using kmeans clustering
        #     plot.test <- dplyr::select(list.data.clean,Nematode,dpi,replicate,gene_id,FPKM_ratio_mean) %>%
        #                  dplyr::filter(gene_id %in% unique(filter(model.data,significance<0.001)$gene_id)) %>%
        #                  group_by(Nematode,gene_id,dpi) %>%
        #                  summarise(FPKM_ratio_mean=median(FPKM_ratio_mean)) %>%
        #                  data.frame() %>%
        #                  mutate(SampleID=paste(Nematode,dpi,sep="_")) %>%
        #                  dplyr::select(-Nematode,-dpi) %>%
        #                  tidyr::spread(key=SampleID,value=FPKM_ratio_mean) %>%
        #                  tibble::column_to_rownames(var='gene_id')
        # 
        # 
        #     kclusts <- dplyr::tibble(k=1:20) %>%
        #                dplyr::mutate(kclust = map(k, ~kmeans(plot.test, .x,iter.max=1000,nstart=50)),
        #                              tidied = map(kclust, tidy),
        #                              glanced = map(kclust, glance),
        #                              augmented = map(kclust, augment, plot.test)
        #                             )
        # 
        #     ###unwrap clustering
        # 
        #         clusters <- kclusts %>%
        #                     unnest(tidied)
        # 
        #         assignments <- kclusts %>%
        #                        unnest(augmented)
        # 
        #         clusterings <- kclusts %>%
        #                        unnest(glanced)
        # 
        #         ###k=5 seems good
        #         ggplot(clusterings, aes(k, tot.withinss)) +
        #         geom_line()
        # 
        # data.plot <- dplyr::filter(list.data.clean,gene_id %in% unique(filter(model.data,significance<0.001)$gene_id)) %>%
        #              dplyr::select(gene_id,Nematode,dpi,replicate,FPKM_ratio_mean) %>%
        #              merge(dplyr::select(filter(assignments,k==5),.rownames,.cluster),by.x=1,by.y=1) %>%
        #              arrange(.cluster,FPKM_ratio_mean) %>%
        #              mutate(gene=factor(gene_id,levels = unique(gene_id)),dpi=factor(dpi,levels=c(3,6)),Nematode=factor(Nematode,levels = c("MQ","D383","Rookmaker"))) %>%
        #              group_by(dpi,Nematode,gene,.cluster) %>%
        #              summarise(FPKM_ratio_mean=mean(FPKM_ratio_mean))
        # 
        # kmeans.data <- data.plot
        # save(kmeans.data,file="obj_kmeans.data.Rdata")

        load(file="./data/obj_kmeans.data.Rdata")
        data.plot <- kmeans.data
        
        pdf(file="Figure1D-kmeans_DEG.pdf",width=7,height=7)
            ggplot(data.plot,aes(x=dpi,y=gene,fill=FPKM_ratio_mean)) +
            geom_raster() + facet_grid(.cluster~Nematode,scales = "free_y",space = "free_y") + 
            presentation + theme(axis.ticks.y=element_blank(),axis.text.y = element_blank(),strip.text.y = element_text(angle=0)) +
            scale_fill_gradientn(colours=brewer.pal(11,"RdYlBu")[-c(9,10,11)],name="log2(FPKM/mean FPKM)") + 
            scale_x_discrete(expand=c(0,0)) + labs(x="Days post inocculation",y="Transcript")
        dev.off()
            
        data.table <- data.frame(data.plot) %>%
                      filter(!duplicated(gene)) %>%
                      dplyr::select(.cluster,gene) %>%
                      merge(gene.id,by.x=2,by.y=1) %>%
                      arrange(.cluster)
        
        write.xlsx(data.table,file="Supplementary_table5_kmeans_genes.xlsx")       

        
    ################################################################################
    ###RNAseq: enrichment analysis plots (Figure 2)
    ################################################################################
    
        setwd("./data/enrichment")
                   
        files <- dir()[grepl(".csv",dir())]
        
        data.plot <- read.csv(files[2]) %>%
                     arrange(negative_log10_of_adjusted_p_value) %>%
                     mutate(term_name=factor(term_name,levels = unique(term_name))) %>%
                     mutate(fold_enrichment=intersection_size/(term_size/effective_domain_size*query_size))
        
        ggplot(data.plot,aes(y=term_name,x=negative_log10_of_adjusted_p_value,colour=term_size)) +
        geom_point(aes(size=fold_enrichment)) + presentation + scale_color_gradientn(colours=brewer.pal(9,"Reds")[-c(1,2)]) +
        labs(x=expression(bold(-log[10](corrected~bolditalic(P)~value))),y="GO-term") + theme(legend.position = c(0.8,0.3)) +
        xlim(0,5) + geom_vline(xintercept = 1.3,col="blue",size=1.5,lty=2)
               
                        
        
        
    ################################################################################
    ###RNAseq: plots of all DEG (Figure 3)
    ################################################################################

        genes <- readxl::read_xlsx("./data/20220228_D383-specific DEGs showing.xlsx")
        
        pdf(file="specific_genes.pdf",width=5,height=3)
        for(i in 1:nrow(genes)){
            data.plot <- filter(list.data.clean,gene_id==genes$gene_id[i]) %>%
                         mutate(Nematode=factor(Nematode,levels=c("Mock","D383","Rookmaker")))
            
            print(
            ggplot(data.plot,aes(x=dpi,y=FPKM_log,fill=Nematode)) +
            geom_boxplot() + geom_jitter() + facet_grid(~Nematode) +
            presentation + ggtitle(label=paste(genes$gene_description[i],genes$gene_id[i],sep="\n")) + fillScale +
            theme(legend.position = "none",plot.title = element_text(size=14, face="bold")) + labs(y=expression(bold(log[2](FPKM))))
            )
            cat(i/nrow(genes),"")
        }
        dev.off()        


        genes <- c("PGSC0003DMG400010226", "PGSC0003DMG400001550", "PGSC0003DMG400008391", 
                   "PGSC0003DMG401031196", "PGSC0003DMG400009509", "PGSC0003DMG400031519", 
                   "PGSC0003DMG403030532", "PGSC0003DMG400017919", "PGSC0003DMG400014558", 
                   "PGSC0003DMG400027372", "PGSC0003DMG400001274", "PGSC0003DMG400028904", 
                   "PGSC0003DMG400031140", "PGSC0003DMG400000211", "PGSC0003DMG400007387", 
                   "PGSC0003DMG400020608", "PGSC0003DMG400022929", "PGSC0003DMG400026261", 
                   "PGSC0003DMG400003530", "PGSC0003DMG401031741", "PGSC0003DMG400016280", 
                   "PGSC0003DMG400023295", "PGSC0003DMG400040547"
)
        
        
        pdf(file="specific_genes2.pdf",width=4,height=3)
        for(i in 1:length(genes)){
            data.plot <- filter(list.data.clean,gene_id==genes[i]) %>%
                         mutate(Nematode=factor(Nematode,levels=c("Mock","D383","Rookmaker")))
            
            print(
            ggplot(data.plot,aes(x=dpi,y=FPKM_log,fill=Nematode)) +
            geom_boxplot() + geom_jitter() + facet_grid(~Nematode) +
            presentation + ggtitle(label=genes[i]) + fillScale +
            theme(legend.position = "none",plot.title = element_text(size=14, face="bold")) + labs(y=expression(bold(log[2](FPKM))))
            )
            cat(i/length(genes),"")
        }
        dev.off()

  
 
        
      
        