#########normalize and QC

 
    ###RNAseq
        data <- read.delim("./Data/fpkm_genename.txt")
    
        gene.id <- dplyr::select(data,gene_id,gene_name:tf_family)
        
        list.data <- dplyr::select(data,-c(gene_name:tf_family)) %>%
                     tidyr::gather(key=SampleID,value=FPKM,-gene_id) %>%
                     tidyr::separate(SampleID,into=c("Nematode","dpi","replicate"),sep="_",remove=FALSE) %>%
                     dplyr::mutate(dpi=gsub("dpi","",dpi),Nematode=ifelse(Nematode=="D","D383",ifelse(Nematode=="R","Rookmaker","MQ")))
    

    ###Load data
        tmp <- list.data
        
        ###Check detection of reads over samples
            data.plot <- group_by(tmp,gene_id) %>%
                         summarise(n_counts=sum(FPKM>0))
            
            p1 <- ggplot(data.plot,aes(x=n_counts)) +
                  geom_histogram(binwidth=1) + labs(title="Read coverage")
           
            
            
        ###Check detection of reads over samples
            data.plot <- group_by(tmp,gene_id, Nematode !="MQ") %>%
                         summarise(n_counts=sum(FPKM>0)) %>%  
                         group_by(gene_id) %>%
                         filter(sum(n_counts==0)==1) %>%
                         data.frame() %>%
                         filter(!duplicated(gene_id))
            
            data.plot2 <- filter(list.data,gene_id %in% data.plot$gene_id) %>%
                          group_by(gene_id, Nematode !="MQ") %>%
                          mutate(n_counts=sum(FPKM>0)/length(FPKM)) %>% 
                          group_by(gene_id) %>%
                          mutate(n_counts=max(n_counts)) %>%
                          data.frame() %>%
                          filter(n_counts==1)
            
            ggplot(data.plot2,aes(x=dpi,y=log(FPKM),colour=Nematode)) +
            geom_point() + facet_wrap(~gene_id)
             
            
                         
            
             
        ###Check FPKM distribution reads detected in all samples 
            data.plot <- group_by(tmp,gene_id) %>%
                         mutate(n_counts=sum(FPKM>0)) %>%
                         data.frame() %>%
                         filter(n_counts==max(n_counts))
            
            p2 <- ggplot(data.plot,aes(x=FPKM)) +
                  geom_histogram(binwidth=1) + coord_cartesian(xlim=c(0,250)) + labs(title="FPKM")
                     
        ###normalize
            pot_rna <- group_by(tmp,gene_id) %>%
                       mutate(n_counts=sum(FPKM>0)) %>%
                       data.frame() %>%
                       filter(n_counts==max(n_counts)) %>%
                       mutate(FPKM_log=log2(FPKM+1)) %>%
                       group_by(gene_id) %>%
                       mutate(FPKM_ratio_mean=log2(FPKM/mean(FPKM)),FPKM_zscore=((FPKM)-mean(FPKM))/sd(FPKM)) %>%
                       data.frame()
            
        ###plot distribution transformations
            p3 <- ggplot(pot_rna,aes(x=FPKM_zscore)) +
                  geom_histogram(binwidth=0.1) + labs(title="Post-norm FPKM z-score")
    
            p4 <- ggplot(pot_rna,aes(x=FPKM_log)) +
                  geom_histogram(binwidth=0.1) + labs(title="Post-norm FPKM log2")
            
            p5 <- ggplot(pot_rna,aes(x=FPKM_ratio_mean)) +
                  geom_histogram(binwidth=0.1) + labs(title="Post-norm FPKM ratio with mean")
                
        ###Stats for paper/ reporting
            ###unique genes you keep 19590 genes
            length(unique(pot_rna$gene_id))

            ###unique genes pre filter: 39028
            dim(tmp)[1]/length(unique(tmp$SampleID ))
        
    ###Load gene annotation
        list.data.clean <- pot_rna

        save(list.data.clean,file="./obj_list.data.clean.Rdata")
        save(gene.id,file="./obj_gen.id.Rdata")
        write.table(list.data.clean,file="./Normalized_RNAseq.txt",sep="\t",quote=F)
        
        pdf("./Quality_check.pdf",width=8,height=5)
            print(p1); print(p2); print(p3); print(p4); print(p5)
        dev.off()
        
    ###Correlation analysis
        
        rnaseq.check.cor(rename(list.data.clean,sample=SampleID,TPM_log=FPKM_log,TPM=FPKM))
        
        
        # "PGSC0003DMG400007867","gpa2 (maybe)"
        # "PGSC0003DMG400002029","P4"
        
    
################################################################################
###Batch correction
################################################################################
        

    list.data <- list.data.clean
    
    ###Batch correction
        ids <- unique(list.data$gene_id)
        for(i in 1:length(ids)){
            data <- filter(list.data,gene_id==ids[i])
            
            tmp <- summary(lm(FPKM_log~as.factor(replicate)+dpi+Nematode,data=data))[[4]]
            
            list.data <- mutate(list.data,FPKM_log=ifelse(gene_id==ids[i] & replicate==2,FPKM_log-tmp[2,1],FPKM_log)) %>%
                         mutate(FPKM_log=ifelse(gene_id==ids[i] & replicate==3,FPKM_log-tmp[3,1],FPKM_log)) 
            cat(i," ")
        }  
    
        list.data <- select(list.data, -c(FPKM,FPKM_ratio_mean,FPKM_zscore)) %>%
                     group_by(gene_id) %>%
                     mutate(FPKM_ratio_mean=log2((2^FPKM_log)/(mean(2^FPKM_log))),FPKM=2^FPKM_log) %>%
                     data.frame()
     
        save(list.data,file="list.data.tmp.Rdata")


        
            
#############