################################################################################################################################################################
### rnaseq.check.cor(list_df,filename) v1.0
###   returns - the mean of the correlations of sample x with all other samples
###           - a plot of the correlations of sample x with all other samples, with an indication which samples are < 0.9 (saved as pdf) and a heatmap of the correlations (saved as pdf)
###   obligatory input
###    list_df: a list with gene expression data, containing the columns: sample, geneID, TPM_log, TPM
###
###   optional input
###    filename: name to give to the output plot
###    save_dir: location to save the data to
###
###   depends on
###    date.nu

rnaseq.check.cor <- function(list_df,filename){
                             if(missing(list_df)){                                                          stop("Requires a dataframe with expression values")}
                             if(missing(filename)){                                                         filename <- ""}
                             if(sum(c("sample","gene_id","TPM_log","TPM") %in% colnames(list_df)) != 4){    stop("make sure colnames: sample, geneID, TPM_log, TPM are used")}
                             
                             ###dependencies
                                 ##test if required packages are installed
                                 if(!require("tidyverse")){
                                     install.packages("tidyverse")
                                     library("tidyverse")
                                 }

    
                             ###Correlation
                             input <- dplyr::select(list_df,sample,gene_id,TPM) %>%
                                      tidyr::spread(key=sample,value=TPM) %>%
                                      tibble::column_to_rownames("gene_id")
                             input <- data.matrix(input)
                             
                             correlations <- cor(input)
                                 write.table(correlations,file=paste(date.nu(),"_correlations.txt",sep=""),sep="\t",quote=F)
                             correlsum <- apply(correlations,1,mean)

                             if(filename==""){pdf(file=paste(date.nu(),"_correlation.check.pdf",sep=""),width=20,height=9)}
                             if(filename!=""){pdf(file=paste(date.nu(),"_",filename,"_correlation.check.pdf",sep=""),width=20,height=9)}
                                 ###Log2int sample correlations
                                 axis.nu <- axis.output(1,length(correlsum))
                                 par(fig=c(0.3,0.7,0.1,0.9))
                                 plot(sort(correlsum),pch=19,axes=FALSE,xlab="",ylab="",ylim=c(0.75,1))
                                     axis(1,axis.nu[[2]],axis.nu[[2]])
                                     mtext(side=1,text="Samples",cex=1.5,line=3)
                                     axis(2,seq(0.75,1,by=0.05),seq(0.75,1,by=0.05),las=2)
                                     mtext(side=2,text="Mean rho",cex=1.5,line=3)
                                     abline(h=0.9,lwd=2,col="grey",lty=2)
                                     if(sum(correlsum < 0.9) > 0){
                                         for(i in 1:sum(correlsum < 0.9)){
                                             lines(x=c(i,length(correlsum)/4),y=c(sort(correlsum)[sort(correlsum)<0.9][i],seq(0.745,0.895,length=sum(correlsum < 0.9))[i]))
                                             text(x=length(correlsum)/4,y=seq(0.745,0.895,length=sum(correlsum < 0.9))[i],labels=colnames(input)[order(correlsum)][i],pos=4,cex=0.7)
                                         }
                                     }
                                 box()

                                 ###ratio mean sample correlations
                                 input <- dplyr::select(list_df,sample,gene_id,TPM_log) %>%
                                          tidyr::spread(key=sample,value=TPM_log) %>%
                                          tibble::column_to_rownames("gene_id")
                                 input <- data.matrix(input)
                                 
                                 correlations <- cor(input)
                                 colscale <- color.scale(colors.use=c("#8E0152","#C51B7D","#DE77AE","#F1B6DA","#FDE0EF","#F7F7F7","#E6F5D0","#B8E186","#7FBC41","#4D9221","#276419"),
                                                         y.val=as.numeric(correlations))

                                 heatmap(correlations,scale="none",col=colscale,margins=c(10,8))
                                 par(fig=c(0,0.3,0.5,1),new=T)
                                 scale.legend(input=as.numeric(correlations),start.zero=F,col.scale=colscale,line.col="black")
                             dev.off()

                             return(correlsum)
                            }