###linear model for transcriptomics

###input
#lm_data (matrix with transcriptome data)
#lm_factors (dataframe with factors over which data should be explained)
#lm_model (either addtive, interaction, or custom)
#lm_model_custom (in case custom is selected as model, provide the model)

###output
#list with input data, factors, and outcome of the model

###Description
# custom models can be written as lm_factors[,1] + lm_factors[,2] + lm_factors[,1] : lm_factors[,3]


transcriptomics.lm <- function(lm_data,lm_factors,lm_model,lm_model_custom){
                                if(missing(lm_data)){                                       stop("provide data for the linear model")}
                                if(missing(lm_factors)){                                    stop("provide dataframe with the explanatory factors")}
                                if(missing(lm_model)){                                      lm_model <- "additive"}
                                if(!lm_model %in% c("additive","interaction","custom")){    stop("the supported model types are: additive, interaction, and custom")}
                                if(lm_model == "custom"){
                                    if(missing(lm_model_custom)){                           stop("provide a model in the form of: lm_factors[,1] + lm_factors[,2] + lm_factors[,1] : lm_factors[,3]")}
                                }

                                if(length(dim(lm_factors)) == 0){lm_factors <- matrix(lm_factors,ncol=1); colnames(lm_factors) <- "x"}

                                ###constructs the names of the interaction model
                                interaction_names <- function(output){
                                                              name_matrix <- matrix(colnames(output[[2]]),ncol=ncol(output[[2]]),nrow=ncol(output[[2]]))
                                                              tmp <- list()
                                                              lm_names <- NULL

                                                              for(j in 1:ncol(output[[2]])){
                                                                  if(j == 1){
                                                                      tmp[[j]] <- name_matrix
                                                                  } else {
                                                                      tmp[[j]] <- name_matrix[-c(1:(j-1)),-c(1:(j-1))]
                                                                  }
                                                                  if(j < ncol(output[[2]])){
                                                                      for(i in 2:ncol(tmp[[j]])){
                                                                          tmp[[j]][,i] <- paste(paste(tmp[[j]][1:(i-1),i],collapse=":"),tmp[[j]][,i],sep=":")
                                                                      }
                                                                  }
                                                                  tmp[[j]][upper.tri(tmp[[j]])] <- NA
                                                              }
                                                              for(j in 1:ncol(output[[2]])){
                                                                  if(j == 1){
                                                                      lm_names <- c(lm_names,tmp[[j]][,j])
                                                                  } else {
                                                                      for(i in 1:ncol(output[[2]])){
                                                                          if(i <= (ncol(output[[2]])+1-j)){
                                                                              lm_names <- c(lm_names,tmp[[i]][,j])
                                                                          }
                                                                      }
                                                                  }
                                                              }
                                                              lm_names <- lm_names[!is.na(lm_names)]
                                                              return(lm_names)
                                                             }

                                output <- list()
                                    output[[1]] <- lm_data; if(length(rownames(output[[1]])) == 0){rownames(output[[1]]) <- 1:nrow(output[[1]])}
                                    output[[2]] <- lm_factors; if(length(colnames(output[[2]])) == 0){colnames(output[[2]]) <- 1:ncol(output[[2]])}


                                if(lm_model == "additive"){
                                    lm_model_custom <- paste("lm(t(output[[1]]) ~ ",paste(paste("output[[2]][,",1:ncol(output[[2]]),"]",sep=""),collapse=" + "),")",sep="")
                                    tmp <- fast.summ.transcriptomics(eval(parse(text=lm_model_custom)))
                                        colnames(tmp) <- c(paste("significance_",colnames(output[[2]]),sep=""),paste("effect_",colnames(output[[2]]),sep=""))
                                }
                                if(lm_model == "interaction"){
                                    lm_model_custom <- paste("lm(t(output[[1]]) ~ ",paste(paste("output[[2]][,",1:ncol(output[[2]]),"]",sep=""),collapse=" * "),")",sep="")
                                    tmp <- fast.summ.transcriptomics(eval(parse(text=lm_model_custom)))
                                        colnames(tmp) <- c(paste("significance_",interaction_names(output),sep=""),paste("effect_",interaction_names(output),sep=""))
                                }
                                if(lm_model == "custom"){
                                    lm_model_custom <- paste("lm(t(output[[1]]) ~ ",lm_model_custom,")",sep="")
                                    tmp <- fast.summ.transcriptomics(eval(parse(text=lm_model_custom)))
                                }



                                output[[3]] <- tmp

                                names(output) <- c("data","factors","model")

                                return(output)
                               }
